# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2014-2020 messense

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatKF(BaseWeChatAPI):
    """
    微信客服接口

    https://work.weixin.qq.com/api/doc/90000/90135/94670
    """

    def sync_msg(self, token, open_kfid, cursor="", limit=1000):
        """
        微信客户发送的消息、接待人员在企业微信回复的消息、发送消息接口发送失败事件（如被用户拒收）
        、客户点击菜单消息的回复消息，可以通过该接口获取具体的消息内容和事件。不支持读取通过发送消息接口发送的消息。
        支持的消息类型：文本、图片、语音、视频、文件、位置、链接、名片、小程序、事件。


        :param token: 回调事件返回的token字段，10分钟内有效；可不填，如果不填接口有严格的频率限制。不多于128字节
        :param open_kfid: 客服帐号ID
        :param cursor: 上一次调用时返回的next_cursor，第一次拉取可以不填。不多于64字节
        :param limit: 期望请求的数据量，默认值和最大值都为1000。
        注意：可能会出现返回条数少于limit的情况，需结合返回的has_more字段判断是否继续请求。
        :return: 接口调用结果
        """
        data = {"token": token, "cursor": cursor, "limit": limit, "open_kfid": open_kfid}
        return self._post("kf/sync_msg", data=data)

    def get_service_state(self, open_kfid, external_userid):
        """
        获取会话状态

        ID	状态	说明
        0	未处理	新会话接入。可选择：1.直接用API自动回复消息。2.放进待接入池等待接待人员接待。3.指定接待人员进行接待
        1	由智能助手接待	可使用API回复消息。可选择转入待接入池或者指定接待人员处理。
        2	待接入池排队中	在待接入池中排队等待接待人员接入。可选择转为指定人员接待
        3	由人工接待	人工接待中。可选择结束会话
        4	已结束	会话已经结束。不允许变更会话状态，等待用户重新发起咨询

        :param open_kfid: 客服帐号ID
        :param external_userid: 微信客户的external_userid
        :return: 接口调用结果
        """
        data = {
            "open_kfid": open_kfid,
            "external_userid": external_userid,
        }
        return self._post("kf/service_state/get", data=data)

    def trans_service_state(self, open_kfid, external_userid, service_state, servicer_userid=""):
        """
        变更会话状态

        :param open_kfid: 客服帐号ID
        :param external_userid: 微信客户的external_userid
        :param service_state: 当前的会话状态，状态定义参考概述中的表格
        :return: 接口调用结果
        """
        data = {
            "open_kfid": open_kfid,
            "external_userid": external_userid,
            "service_state": service_state,
        }
        if servicer_userid:
            data["servicer_userid"] = servicer_userid
        return self._post("kf/service_state/trans", data=data)

    def get_servicer_list(self, open_kfid):
        """
        获取接待人员列表

        :param open_kfid: 客服帐号ID
        :return: 接口调用结果
        """
        data = {
            "open_kfid": open_kfid,
        }
        return self._get("kf/servicer/list", params=data)

    def add_servicer(self, open_kfid, userid_list):
        """
        添加接待人员
        添加指定客服帐号的接待人员。

        :param open_kfid: 客服帐号ID
        :param userid_list: 接待人员userid列表
        :return: 接口调用结果
        """
        if not isinstance(userid_list, list):
            userid_list = [userid_list]

        data = {
            "open_kfid": open_kfid,
            "userid_list": userid_list,
        }
        return self._post("kf/servicer/add", data=data)

    def del_servicer(self, open_kfid, userid_list):
        """
        删除接待人员
        从客服帐号删除接待人员

        :param open_kfid: 客服帐号ID
        :param userid_list: 接待人员userid列表
        :return: 接口调用结果
        """
        if not isinstance(userid_list, list):
            userid_list = [userid_list]

        data = {
            "open_kfid": open_kfid,
            "userid_list": userid_list,
        }
        return self._post("kf/servicer/del", data=data)

    def batchget_customer(self, external_userid_list):
        """
        客户基本信息获取

        :param external_userid_list: external_userid列表
        :return: 接口调用结果
        """
        if not isinstance(external_userid_list, list):
            external_userid_list = [external_userid_list]

        data = {
            "external_userid_list": external_userid_list,
        }
        return self._post("kf/customer/batchget", data=data)

    def get_account_list(self):
        """
        获取客服帐号列表

        :return: 接口调用结果
        """
        return self._get("kf/account/list")

    def add_contact_way(self, open_kfid, scene):
        """
        获取客服帐号链接

        :param open_kfid: 	客服帐号ID
        :param scene: 场景值，字符串类型，由开发者自定义。不多于32字节;字符串取值范围(正则表达式)：[0-9a-zA-Z_-]*
        :return: 接口调用结果
        """
        data = {"open_kfid": open_kfid, "scene": scene}
        return self._post("kf/add_contact_way", data=data)

    def get_upgrade_service_config(self):
        """
        获取配置的专员与客户群

        :return: 接口调用结果
        """
        return self._get("kf/customer/get_upgrade_service_config")

    def upgrade_service(self, open_kfid, external_userid, service_type, member=None, groupchat=None):
        """
        为客户升级为专员或客户群服务

        :param open_kfid: 	客服帐号ID
        :param external_userid: 微信客户的external_userid
        :param service_type: 表示是升级到专员服务还是客户群服务。1:专员服务。2:客户群服务
        :param member: 推荐的服务专员，type等于1时有效
        :param groupchat: 推荐的客户群，type等于2时有效
        :return: 接口调用结果
        """

        data = {
            "open_kfid": open_kfid,
            "external_userid": external_userid,
            "type": service_type,
        }
        if service_type == 1:
            data["member"] = member
        else:
            data["groupchat"] = groupchat
        return self._post("kf/customer/upgrade_service", data=data)

    def cancel_upgrade_service(self, open_kfid, external_userid):
        """
        为客户取消推荐

        :param open_kfid: 	客服帐号ID
        :param external_userid: 微信客户的external_userid
        :return: 接口调用结果
        """

        data = {"open_kfid": open_kfid, "external_userid": external_userid}
        return self._post("kf/customer/cancel_upgrade_service", data=data)

    def send_msg_on_event(self, code, msgtype, msg_content, msgid=None):
        """
        当特定的事件回调消息包含code字段，可以此code为凭证，调用该接口给用户发送相应事件场景下的消息，如客服欢迎语。
        支持发送消息类型：文本、菜单消息。

        :param code: 事件响应消息对应的code。通过事件回调下发，仅可使用一次。
        :param msgtype: 消息类型。对不同的msgtype，有相应的结构描述，详见消息类型
        :param msg_content: 目前支持文本与菜单消息，具体查看文档
        :param msgid: 消息ID。如果请求参数指定了msgid，则原样返回，否则系统自动生成并返回。不多于32字节；
                      字符串取值范围(正则表达式)：[0-9a-zA-Z_-]*
        :return: 接口调用结果
        """

        data = {"code": code, "msgtype": msgtype}
        if msgid:
            data["msgid"] = msgid
        data.update(msg_content)
        return self._post("kf/send_msg_on_event", data=data)

    def get_corp_statistic(self, start_time, end_time, open_kfid=None):
        """
        获取「客户数据统计」企业汇总数据

        :param start_time: 开始时间
        :param end_time: 结束时间
        :param open_kfid: 	客服帐号ID
        :return: 接口调用结果
        """
        data = {"open_kfid": open_kfid, "start_time": start_time, "end_time": end_time}
        return self._post("kf/get_corp_statistic", data=data)

    def get_servicer_statistic(self, start_time, end_time, open_kfid=None, servicer_userid=None):
        """
        获取「客户数据统计」接待人员明细数据

        :param start_time: 开始时间
        :param end_time: 结束时间
        :param open_kfid: 	客服帐号ID
        :param servicer_userid: 接待人员
        :return: 接口调用结果
        """
        data = {
            "open_kfid": open_kfid,
            "servicer_userid": servicer_userid,
            "start_time": start_time,
            "end_time": end_time,
        }
        return self._post("kf/get_servicer_statistic", data=data)

    def account_update(self, open_kfid, name, media_id):
        """
        修改客服账号

        :param open_kfid: 客服帐号ID
        :param name: 客服名称
        :param media_id: 客服头像临时素材

        :return: 接口调用结果
        """
        data = {"open_kfid": open_kfid, "name": name, "media_id": media_id}
        return self._post("kf/account/update", data=data)



import asyncio
import os
import uuid
import aiohttp
import dingtalk_stream
import threading

from astrbot.api.platform import (
    Platform,
    AstrBotMessage,
    MessageMember,
    MessageType,
    PlatformMetadata,
)
from astrbot.api.event import MessageChain
from astrbot.api.message_components import Image, Plain, At
from astrbot.core.platform.astr_message_event import MessageSesion
from .dingtalk_event import DingtalkMessageEvent
from ...register import register_platform_adapter
from astrbot import logger
from dingtalk_stream import AckMessage
from astrbot.core.utils.io import download_file
from astrbot.core.utils.astrbot_path import get_astrbot_data_path


class MyEventHandler(dingtalk_stream.EventHandler):
    async def process(self, event: dingtalk_stream.EventMessage):
        print(
            "2",
            event.headers.event_type,
            event.headers.event_id,
            event.headers.event_born_time,
            event.data,
        )
        return AckMessage.STATUS_OK, "OK"


@register_platform_adapter("dingtalk", "钉钉机器人官方 API 适配器")
class DingtalkPlatformAdapter(Platform):
    def __init__(
        self, platform_config: dict, platform_settings: dict, event_queue: asyncio.Queue
    ) -> None:
        super().__init__(event_queue)

        self.config = platform_config

        self.unique_session = platform_settings["unique_session"]

        self.client_id = platform_config["client_id"]
        self.client_secret = platform_config["client_secret"]

        class AstrCallbackClient(dingtalk_stream.ChatbotHandler):
            async def process(self_, message: dingtalk_stream.CallbackMessage):
                logger.debug(f"dingtalk: {message.data}")
                im = dingtalk_stream.ChatbotMessage.from_dict(message.data)
                abm = await self.convert_msg(im)
                await self.handle_msg(abm)

                return AckMessage.STATUS_OK, "OK"

        self.client = AstrCallbackClient()

        credential = dingtalk_stream.Credential(self.client_id, self.client_secret)
        client = dingtalk_stream.DingTalkStreamClient(credential, logger=logger)
        client.register_all_event_handler(MyEventHandler())
        client.register_callback_handler(
            dingtalk_stream.ChatbotMessage.TOPIC, self.client
        )
        self.client_ = client  # 用于 websockets 的 client

    async def send_by_session(
        self, session: MessageSesion, message_chain: MessageChain
    ):
        raise NotImplementedError("钉钉机器人适配器不支持 send_by_session")

    def meta(self) -> PlatformMetadata:
        return PlatformMetadata(
            name="dingtalk",
            description="钉钉机器人官方 API 适配器",
            id=self.config.get("id"),
        )

    async def convert_msg(
        self, message: dingtalk_stream.ChatbotMessage
    ) -> AstrBotMessage:
        abm = AstrBotMessage()
        abm.message = []
        abm.message_str = ""
        abm.timestamp = int(message.create_at / 1000)
        abm.type = (
            MessageType.GROUP_MESSAGE
            if message.conversation_type == "2"
            else MessageType.FRIEND_MESSAGE
        )
        abm.sender = MessageMember(
            user_id=message.sender_id, nickname=message.sender_nick
        )
        abm.self_id = message.chatbot_user_id
        abm.message_id = message.message_id
        abm.raw_message = message

        if abm.type == MessageType.GROUP_MESSAGE:
            if message.is_in_at_list:
                abm.message.append(At(qq=abm.self_id))
            abm.group_id = message.conversation_id
            if self.unique_session:
                abm.session_id = abm.sender.user_id
            else:
                abm.session_id = abm.group_id
        else:
            abm.session_id = abm.sender.user_id

        message_type: str = message.message_type
        match message_type:
            case "text":
                abm.message_str = message.text.content.strip()
                abm.message.append(Plain(abm.message_str))
            case "richText":
                rtc: dingtalk_stream.RichTextContent = message.rich_text_content
                contents: list[dict] = rtc.rich_text_list
                for content in contents:
                    plains = ""
                    if "text" in content:
                        plains += content["text"]
                        abm.message.append(Plain(plains))
                    elif "type" in content and content["type"] == "picture":
                        f_path = await self.download_ding_file(
                            content["downloadCode"],
                            message.robot_code,
                            "jpg",
                        )
                        abm.message.append(Image.fromFileSystem(f_path))
            case "audio":
                pass

        return abm  # 别忘了返回转换后的消息对象

    async def download_ding_file(
        self, download_code: str, robot_code: str, ext: str
    ) -> str:
        """下载钉钉文件

        :param access_token: 钉钉机器人的 access_token
        :param download_code: 下载码
        :param robot_code: 机器人码
        :param ext: 文件后缀
        :return: 文件路径
        """
        access_token = await self.get_access_token()
        headers = {
            "x-acs-dingtalk-access-token": access_token,
        }
        payload = {
            "downloadCode": download_code,
            "robotCode": robot_code,
        }
        temp_dir = os.path.join(get_astrbot_data_path(), "temp")
        f_path = os.path.join(temp_dir, f"dingtalk_file_{uuid.uuid4()}.{ext}")
        async with aiohttp.ClientSession() as session:
            async with session.post(
                "https://api.dingtalk.com/v1.0/robot/messageFiles/download",
                headers=headers,
                json=payload,
            ) as resp:
                if resp.status != 200:
                    logger.error(
                        f"下载钉钉文件失败: {resp.status}, {await resp.text()}"
                    )
                    return None
                resp_data = await resp.json()
                download_url = resp_data["data"]["downloadUrl"]
                await download_file(download_url, f_path)
        return f_path

    async def get_access_token(self) -> str:
        payload = {
            "appKey": self.client_id,
            "appSecret": self.client_secret,
        }
        async with aiohttp.ClientSession() as session:
            async with session.post(
                "https://api.dingtalk.com/v1.0/oauth2/accessToken",
                json=payload,
            ) as resp:
                if resp.status != 200:
                    logger.error(
                        f"获取钉钉机器人 access_token 失败: {resp.status}, {await resp.text()}"
                    )
                    return None
                return (await resp.json())["data"]["accessToken"]

    async def handle_msg(self, abm: AstrBotMessage):
        event = DingtalkMessageEvent(
            message_str=abm.message_str,
            message_obj=abm,
            platform_meta=self.meta(),
            session_id=abm.session_id,
            client=self.client,
        )

        self._event_queue.put_nowait(event)

    async def run(self):
        # await self.client_.start()
        # 钉钉的 SDK 并没有实现真正的异步，start() 里面有堵塞方法。
        def start_client(loop: asyncio.AbstractEventLoop):
            try:
                self._shutdown_event = threading.Event()
                task = loop.create_task(self.client_.start())
                self._shutdown_event.wait()
                if task.done():
                    task.result()
            except Exception as e:
                if "Graceful shutdown" in str(e):
                    logger.info("钉钉适配器已被优雅地关闭")
                    return
                logger.error(f"钉钉机器人启动失败: {e}")

        loop = asyncio.get_event_loop()
        await loop.run_in_executor(None, start_client, loop)

    async def terminate(self):
        def monkey_patch_close():
            raise Exception("Graceful shutdown")

        self.client_.open_connection = monkey_patch_close
        await self.client_.websocket.close(code=1000, reason="Graceful shutdown")
        self._shutdown_event.set()

    def get_client(self):
        return self.client



import json
from astrbot.core import logger
from aiohttp import ClientSession
from typing import Dict, List, Any, AsyncGenerator


class DifyAPIClient:
    def __init__(self, api_key: str, api_base: str = "https://api.dify.ai/v1"):
        self.api_key = api_key
        self.api_base = api_base
        self.session = ClientSession(trust_env=True)
        self.headers = {
            "Authorization": f"Bearer {self.api_key}",
        }

    async def chat_messages(
        self,
        inputs: Dict,
        query: str,
        user: str,
        response_mode: str = "streaming",
        conversation_id: str = "",
        files: List[Dict[str, Any]] = [],
        timeout: float = 60,
    ) -> AsyncGenerator[Dict[str, Any], None]:
        url = f"{self.api_base}/chat-messages"
        payload = locals()
        payload.pop("self")
        payload.pop("timeout")
        logger.info(f"chat_messages payload: {payload}")
        async with self.session.post(
            url, json=payload, headers=self.headers, timeout=timeout
        ) as resp:
            if resp.status != 200:
                text = await resp.text()
                raise Exception(f"chat_messages 请求失败：{resp.status}. {text}")

            buffer = ""
            while True:
                # 保持原有的8192字节限制，防止数据过大导致高水位报错
                chunk = await resp.content.read(8192)
                if not chunk:
                    break

                buffer += chunk.decode("utf-8")
                blocks = buffer.split("\n\n")

                # 处理完整的数据块
                for block in blocks[:-1]:
                    if block.strip() and block.startswith("data:"):
                        try:
                            json_str = block[5:]  # 移除 "data:" 前缀
                            json_obj = json.loads(json_str)
                            yield json_obj
                        except json.JSONDecodeError as e:
                            logger.error(f"JSON解析错误: {str(e)}")
                            logger.error(f"原始数据块: {json_str}")

                # 保留最后一个可能不完整的块
                buffer = blocks[-1] if blocks else ""

    async def workflow_run(
        self,
        inputs: Dict,
        user: str,
        response_mode: str = "streaming",
        files: List[Dict[str, Any]] = [],
        timeout: float = 60,
    ):
        url = f"{self.api_base}/workflows/run"
        payload = locals()
        payload.pop("self")
        payload.pop("timeout")
        logger.info(f"workflow_run payload: {payload}")
        async with self.session.post(
            url, json=payload, headers=self.headers, timeout=timeout
        ) as resp:
            if resp.status != 200:
                text = await resp.text()
                raise Exception(f"workflow_run 请求失败：{resp.status}. {text}")

            buffer = ""
            while True:
                # 保持原有的8192字节限制，防止数据过大导致高水位报错
                chunk = await resp.content.read(8192)
                if not chunk:
                    break

                buffer += chunk.decode("utf-8")
                blocks = buffer.split("\n\n")

                # 处理完整的数据块
                for block in blocks[:-1]:
                    if block.strip() and block.startswith("data:"):
                        try:
                            json_str = block[5:]  # 移除 "data:" 前缀
                            json_obj = json.loads(json_str)
                            yield json_obj
                        except json.JSONDecodeError as e:
                            logger.error(f"JSON解析错误: {str(e)}")
                            logger.error(f"原始数据块: {json_str}")

                # 保留最后一个可能不完整的块
                buffer = blocks[-1] if blocks else ""

    async def file_upload(
        self,
        file_path: str,
        user: str,
    ) -> Dict[str, Any]:
        url = f"{self.api_base}/files/upload"
        payload = {
            "user": user,
            "file": open(file_path, "rb"),
        }
        async with self.session.post(url, data=payload, headers=self.headers) as resp:
            return await resp.json()  # {"id": "xxx", ...}

    async def close(self):
        await self.session.close()

    async def get_chat_convs(self, user: str, limit: int = 20):
        # conversations. GET
        url = f"{self.api_base}/conversations"
        payload = {
            "user": user,
            "limit": limit,
        }
        async with self.session.get(url, params=payload, headers=self.headers) as resp:
            return await resp.json()

    async def delete_chat_conv(self, user: str, conversation_id: str):
        # conversation. DELETE
        url = f"{self.api_base}/conversations/{conversation_id}"
        payload = {
            "user": user,
        }
        async with self.session.delete(url, json=payload, headers=self.headers) as resp:
            return await resp.json()

    async def rename(
        self, conversation_id: str, name: str, user: str, auto_generate: bool = False
    ):
        # /conversations/:conversation_id/name
        url = f"{self.api_base}/conversations/{conversation_id}/name"
        payload = {
            "user": user,
            "name": name,
            "auto_generate": auto_generate,
        }
        async with self.session.post(url, json=payload, headers=self.headers) as resp:
            return await resp.json()



import pytest
import logging
import os
import asyncio
from astrbot.core.pipeline.scheduler import PipelineScheduler, PipelineContext
from astrbot.core.star import PluginManager
from astrbot.core.config.astrbot_config import AstrBotConfig
from astrbot.core.platform.astr_message_event import AstrMessageEvent
from astrbot.core.platform.astrbot_message import (
    AstrBotMessage,
    MessageMember,
    MessageType,
)
from astrbot.core.message.message_event_result import MessageChain, ResultContentType
from astrbot.core.message.components import Plain, At
from astrbot.core.platform.platform_metadata import PlatformMetadata
from astrbot.core.platform.manager import PlatformManager
from astrbot.core.provider.manager import ProviderManager
from astrbot.core.db.sqlite import SQLiteDatabase
from astrbot.core.star.context import Context
from asyncio import Queue

SESSION_ID_IN_WHITELIST = "test_sid_wl"
SESSION_ID_NOT_IN_WHITELIST = "test_sid"
TEST_LLM_PROVIDER = {
    "id": "zhipu_default",
    "type": "openai_chat_completion",
    "enable": True,
    "key": [os.getenv("ZHIPU_API_KEY")],
    "api_base": "https://open.bigmodel.cn/api/paas/v4/",
    "model_config": {
        "model": "glm-4-flash",
    },
}

TEST_COMMANDS = [
    ["help", "已注册的 AstrBot 内置指令"],
    ["tool ls", "函数工具"],
    ["tool on websearch", "激活工具"],
    ["tool off websearch", "停用工具"],
    ["plugin", "已加载的插件"],
    ["t2i", "文本转图片模式"],
    ["sid", "此 ID 可用于设置会话白名单。"],
    ["op test_op", "授权成功。"],
    ["deop test_op", "取消授权成功。"],
    ["wl test_platform:FriendMessage:test_sid_wl2", "添加白名单成功。"],
    ["dwl test_platform:FriendMessage:test_sid_wl2", "删除白名单成功。"],
    ["provider", "当前载入的 LLM 提供商"],
    ["reset", "重置成功"],
    # ["model", "查看、切换提供商模型列表"],
    ["history", "历史记录："],
    ["key", "当前 Key"],
    ["persona", "[Persona]"],
]


class FakeAstrMessageEvent(AstrMessageEvent):
    def __init__(self, abm: AstrBotMessage = None):
        meta = PlatformMetadata("test_platform", "test")
        super().__init__(
            message_str=abm.message_str,
            message_obj=abm,
            platform_meta=meta,
            session_id=abm.session_id,
        )

    async def send(self, message: MessageChain):
        await super().send(message)

    @staticmethod
    def create_fake_event(
        message_str: str,
        session_id: str = "test_sid",
        is_at: bool = False,
        is_group: bool = False,
        sender_id: str = "123456",
    ):
        abm = AstrBotMessage()
        abm.message_str = message_str
        abm.group_id = "test"
        abm.message = [Plain(message_str)]
        if is_at:
            abm.message.append(At(qq="bot"))
        abm.self_id = "bot"
        abm.sender = MessageMember(sender_id, "mika")
        abm.timestamp = 1234567890
        abm.message_id = "test"
        abm.session_id = session_id
        if is_group:
            abm.type = MessageType.GROUP_MESSAGE
        else:
            abm.type = MessageType.FRIEND_MESSAGE
        return FakeAstrMessageEvent(abm)


@pytest.fixture(scope="module")
def event_queue():
    return Queue()


@pytest.fixture(scope="module")
def config():
    cfg = AstrBotConfig()
    cfg["platform_settings"]["id_whitelist"] = [
        "test_platform:FriendMessage:test_sid_wl",
        "test_platform:GroupMessage:test_sid_wl",
    ]
    cfg["admins_id"] = ["123456"]
    cfg["content_safety"]["internal_keywords"]["extra_keywords"] = ["^TEST_NEGATIVE"]
    cfg["provider"] = [TEST_LLM_PROVIDER]
    return cfg


@pytest.fixture(scope="module")
def db():
    return SQLiteDatabase("data/data_v3.db")


@pytest.fixture(scope="module")
def platform_manager(event_queue, config):
    return PlatformManager(config, event_queue)


@pytest.fixture(scope="module")
def provider_manager(config, db):
    return ProviderManager(config, db)


@pytest.fixture(scope="module")
def star_context(event_queue, config, db, platform_manager, provider_manager):
    star_context = Context(event_queue, config, db, provider_manager, platform_manager)
    return star_context


@pytest.fixture(scope="module")
def plugin_manager(star_context, config):
    plugin_manager = PluginManager(star_context, config)
    # await plugin_manager.reload()
    asyncio.run(plugin_manager.reload())
    return plugin_manager


@pytest.fixture(scope="module")
def pipeline_context(config, plugin_manager):
    return PipelineContext(config, plugin_manager)


@pytest.fixture(scope="module")
def pipeline_scheduler(pipeline_context):
    return PipelineScheduler(pipeline_context)


@pytest.mark.asyncio
async def test_platform_initialization(platform_manager: PlatformManager):
    await platform_manager.initialize()


@pytest.mark.asyncio
async def test_provider_initialization(provider_manager: ProviderManager):
    await provider_manager.initialize()


@pytest.mark.asyncio
async def test_pipeline_scheduler_initialization(pipeline_scheduler: PipelineScheduler):
    await pipeline_scheduler.initialize()


@pytest.mark.asyncio
async def test_pipeline_wakeup(pipeline_scheduler: PipelineScheduler, caplog):
    """测试唤醒"""
    # 群聊无 @ 无指令
    caplog.clear()
    mock_event = FakeAstrMessageEvent.create_fake_event("test", is_group=True)
    with caplog.at_level(logging.DEBUG):
        await pipeline_scheduler.execute(mock_event)
    assert any(
        "执行阶段 WhitelistCheckStage" not in message for message in caplog.messages
    )
    # 群聊有 @ 无指令
    mock_event = FakeAstrMessageEvent.create_fake_event(
        "test", is_group=True, is_at=True
    )
    with caplog.at_level(logging.DEBUG):
        await pipeline_scheduler.execute(mock_event)
    assert any("执行阶段 WhitelistCheckStage" in message for message in caplog.messages)
    # 群聊有指令
    mock_event = FakeAstrMessageEvent.create_fake_event(
        "/help", is_group=True, session_id=SESSION_ID_IN_WHITELIST
    )
    await pipeline_scheduler.execute(mock_event)
    assert mock_event._has_send_oper is True


@pytest.mark.asyncio
async def test_pipeline_wl(
    pipeline_scheduler: PipelineScheduler, config: AstrBotConfig, caplog
):
    caplog.clear()
    mock_event = FakeAstrMessageEvent.create_fake_event(
        "test", SESSION_ID_IN_WHITELIST, sender_id="123"
    )
    with caplog.at_level(logging.INFO):
        await pipeline_scheduler.execute(mock_event)
    assert any(
        "不在会话白名单中，已终止事件传播。" not in message
        for message in caplog.messages
    ), "日志中未找到预期的消息"

    mock_event = FakeAstrMessageEvent.create_fake_event("test", sender_id="123")
    with caplog.at_level(logging.INFO):
        await pipeline_scheduler.execute(mock_event)
    assert any(
        "不在会话白名单中，已终止事件传播。" in message for message in caplog.messages
    ), "日志中未找到预期的消息"


@pytest.mark.asyncio
async def test_pipeline_content_safety(pipeline_scheduler: PipelineScheduler, caplog):
    # 测试默认屏蔽词
    caplog.clear()
    mock_event = FakeAstrMessageEvent.create_fake_event(
        "色情", session_id=SESSION_ID_IN_WHITELIST
    )  # 测试需要。
    with caplog.at_level(logging.INFO):
        await pipeline_scheduler.execute(mock_event)
    assert any("内容安全检查不通过" in message for message in caplog.messages), (
        "日志中未找到预期的消息"
    )
    # 测试额外屏蔽词
    mock_event = FakeAstrMessageEvent.create_fake_event(
        "TEST_NEGATIVE", session_id=SESSION_ID_IN_WHITELIST
    )
    with caplog.at_level(logging.INFO):
        await pipeline_scheduler.execute(mock_event)
    assert any("内容安全检查不通过" in message for message in caplog.messages), (
        "日志中未找到预期的消息"
    )
    mock_event = FakeAstrMessageEvent.create_fake_event(
        "_TEST_NEGATIVE", session_id=SESSION_ID_IN_WHITELIST
    )
    with caplog.at_level(logging.INFO):
        await pipeline_scheduler.execute(mock_event)
    assert any("内容安全检查不通过" not in message for message in caplog.messages)
    # TODO: 测试 百度AI 的内容安全检查


@pytest.mark.asyncio
async def test_pipeline_llm(pipeline_scheduler: PipelineScheduler, caplog):
    caplog.clear()
    mock_event = FakeAstrMessageEvent.create_fake_event(
        "just reply me `OK`", session_id=SESSION_ID_IN_WHITELIST
    )
    with caplog.at_level(logging.DEBUG):
        await pipeline_scheduler.execute(mock_event)
    assert any("请求 LLM" in message for message in caplog.messages)
    assert mock_event.get_result() is not None
    assert mock_event.get_result().result_content_type == ResultContentType.LLM_RESULT


@pytest.mark.asyncio
async def test_pipeline_websearch(pipeline_scheduler: PipelineScheduler, caplog):
    caplog.clear()
    mock_event = FakeAstrMessageEvent.create_fake_event(
        "help me search the latest OpenAI news", session_id=SESSION_ID_IN_WHITELIST
    )
    with caplog.at_level(logging.DEBUG):
        await pipeline_scheduler.execute(mock_event)
    assert any("请求 LLM" in message for message in caplog.messages)
    assert any(
        "web_searcher - search_from_search_engine" in message
        for message in caplog.messages
    )


@pytest.mark.asyncio
async def test_commands(pipeline_scheduler: PipelineScheduler, caplog):
    for command in TEST_COMMANDS:
        caplog.clear()
        mock_event = FakeAstrMessageEvent.create_fake_event(
            command[0], session_id=SESSION_ID_IN_WHITELIST
        )
        with caplog.at_level(logging.DEBUG):
            await pipeline_scheduler.execute(mock_event)
        # assert any("执行阶段 ProcessStage" in message for message in caplog.messages)
        assert any(command[1] in message for message in caplog.messages)



from .network_strategy import NetworkRenderStrategy
from .local_strategy import LocalRenderStrategy
from astrbot.core.log import LogManager

logger = LogManager.GetLogger(log_name="astrbot")


class HtmlRenderer:
    def __init__(self, endpoint_url: str = None):
        self.network_strategy = NetworkRenderStrategy(endpoint_url)
        self.local_strategy = LocalRenderStrategy()

    def set_network_endpoint(self, endpoint_url: str):
        """设置 t2i 的网络端点。"""
        logger.info("文本转图像服务接口: " + endpoint_url)
        self.network_strategy.set_endpoint(endpoint_url)

    async def render_custom_template(
        self, tmpl_str: str, tmpl_data: dict, return_url: bool = False
    ):
        """使用自定义文转图模板。该方法会通过网络调用 t2i 终结点图文渲染API。
        @param tmpl_str: HTML Jinja2 模板。
        @param tmpl_data: jinja2 模板数据。

        @return: 图片 URL 或者文件路径，取决于 return_url 参数。

        @example: 参见 https://astrbot.app 插件开发部分。
        """
        local = locals()
        local.pop("self")
        return await self.network_strategy.render_custom_template(**local)

    async def render_t2i(
        self, text: str, use_network: bool = True, return_url: bool = False
    ):
        """使用默认文转图模板。"""
        if use_network:
            try:
                return await self.network_strategy.render(text, return_url=return_url)
            except BaseException as e:
                logger.error(
                    f"Failed to render image via AstrBot API: {e}. Falling back to local rendering."
                )
                return await self.local_strategy.render(text)
        else:
            return await self.local_strategy.render(text)



from .platform import Platform
from .astr_message_event import AstrMessageEvent
from .platform_metadata import PlatformMetadata
from .astrbot_message import AstrBotMessage, MessageMember, MessageType, Group

__all__ = [
    "Platform",
    "AstrMessageEvent",
    "PlatformMetadata",
    "AstrBotMessage",
    "MessageMember",
    "MessageType",
    "Group",
]



import uuid
import base64
import json
import os
import traceback
import asyncio
import aiohttp
import requests
from ..provider import TTSProvider
from ..entities import ProviderType
from ..register import register_provider_adapter
from astrbot import logger

@register_provider_adapter(
    "volcengine_tts", "火山引擎 TTS", provider_type=ProviderType.TEXT_TO_SPEECH
)
class ProviderVolcengineTTS(TTSProvider):
    def __init__(self, provider_config: dict, provider_settings: dict) -> None:
        super().__init__(provider_config, provider_settings)
        self.api_key = provider_config.get("api_key", "")
        self.appid = provider_config.get("appid", "")
        self.cluster = provider_config.get("volcengine_cluster", "")
        self.voice_type = provider_config.get("volcengine_voice_type", "")
        self.speed_ratio = provider_config.get("volcengine_speed_ratio", 1.0)
        self.api_base = provider_config.get("api_base", f"https://openspeech.bytedance.com/api/v1/tts")
        self.timeout = provider_config.get("timeout", 20)

    def _build_request_payload(self, text: str) -> dict:
        return {
            "app": {
                "appid": self.appid,
                "token": self.api_key,
                "cluster": self.cluster
            },
            "user": {
                "uid": str(uuid.uuid4())  
            },
            "audio": {
                "voice_type": self.voice_type,
                "encoding": "mp3",
                "speed_ratio": self.speed_ratio,
                "volume_ratio": 1.0,
                "pitch_ratio": 1.0,
            },
            "request": {
                "reqid": str(uuid.uuid4()),
                "text": text,
                "text_type": "plain",
                "operation": "query",
                "with_frontend": 1,
                "frontend_type": "unitTson"
            }
        }

    async def get_audio(self, text: str) -> str:
        """异步方法获取语音文件路径"""
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer; {self.api_key}"
        }
        
        payload = self._build_request_payload(text)
        
        logger.debug(f"请求头: {headers}")
        logger.debug(f"请求 URL: {self.api_base}")
        logger.debug(f"请求体: {json.dumps(payload, ensure_ascii=False)[:100]}...")
        
        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(
                    self.api_base,
                    data=json.dumps(payload), 
                    headers=headers,
                    timeout=self.timeout
                ) as response:
                    logger.debug(f"响应状态码: {response.status}")
                    
                    response_text = await response.text()
                    logger.debug(f"响应内容: {response_text[:200]}...")
                    
                    if response.status == 200:
                        resp_data = json.loads(response_text)
                        
                        if "data" in resp_data:
                            audio_data = base64.b64decode(resp_data["data"])
                            
                            os.makedirs("data/temp", exist_ok=True)
                            
                            file_path = f"data/temp/volcengine_tts_{uuid.uuid4()}.mp3"
                            
                            loop = asyncio.get_running_loop()
                            await loop.run_in_executor(
                                None, 
                                lambda: open(file_path, "wb").write(audio_data)
                            )
                            
                            return file_path
                        else:
                            error_msg = resp_data.get("message", "未知错误")
                            raise Exception(f"火山引擎 TTS API 返回错误: {error_msg}")
                    else:
                        raise Exception(f"火山引擎 TTS API 请求失败: {response.status}, {response_text}")
        
        except Exception as e:
            error_details = traceback.format_exc()
            logger.debug(f"火山引擎 TTS 异常详情: {error_details}")
            raise Exception(f"火山引擎 TTS 异常: {str(e)}")


import aiohttp
import random
import astrbot.api.star as star
import astrbot.api.event.filter as filter
from astrbot.api.event import AstrMessageEvent, MessageEventResult
from astrbot.api import llm_tool, logger
from .engines.bing import Bing
from .engines.sogo import Sogo
from .engines.google import Google
from readability import Document
from bs4 import BeautifulSoup
from .engines import HEADERS, USER_AGENTS


@star.register(
    name="astrbot-web-searcher",
    desc="让 LLM 具有网页检索能力",
    author="Soulter",
    version="1.14.514",
)
class Main(star.Star):
    """使用 /websearch on 或者 off 开启或者关闭网页搜索功能"""

    def __init__(self, context: star.Context) -> None:
        self.context = context

        self.bing_search = Bing()
        self.sogo_search = Sogo()
        self.google = Google()

        self.websearch_link = self.context.get_config()["provider_settings"].get(
            "web_search_link", False
        )

    async def initialize(self):
        websearch = self.context.get_config()["provider_settings"]["web_search"]
        if websearch:
            self.context.activate_llm_tool("web_search")
            self.context.activate_llm_tool("fetch_url")
        else:
            self.context.deactivate_llm_tool("web_search")
            self.context.deactivate_llm_tool("fetch_url")

    async def _tidy_text(self, text: str) -> str:
        """清理文本，去除空格、换行符等"""
        return text.strip().replace("\n", " ").replace("\r", " ").replace("  ", " ")

    async def _get_from_url(self, url: str) -> str:
        """获取网页内容"""
        header = HEADERS
        header.update({"User-Agent": random.choice(USER_AGENTS)})
        async with aiohttp.ClientSession(trust_env=True) as session:
            async with session.get(url, headers=header, timeout=6) as response:
                html = await response.text(encoding="utf-8")
                doc = Document(html)
                ret = doc.summary(html_partial=True)
                soup = BeautifulSoup(ret, "html.parser")
                ret = await self._tidy_text(soup.get_text())
                return ret

    @filter.command("websearch")
    async def websearch(self, event: AstrMessageEvent, oper: str = None) -> str:
        websearch = self.context.get_config()["provider_settings"]["web_search"]
        if oper is None:
            status = "开启" if websearch else "关闭"
            event.set_result(
                MessageEventResult().message(
                    "当前网页搜索功能状态："
                    + status
                    + "。使用 /websearch on 或者 off 启用或者关闭。"
                )
            )
            return

        if oper == "on":
            self.context.get_config()["provider_settings"]["web_search"] = True
            self.context.get_config().save_config()
            self.context.activate_llm_tool("web_search")
            self.context.activate_llm_tool("fetch_url")
            event.set_result(MessageEventResult().message("已开启网页搜索功能"))
        elif oper == "off":
            self.context.get_config()["provider_settings"]["web_search"] = False
            self.context.get_config().save_config()
            self.context.deactivate_llm_tool("web_search")
            self.context.deactivate_llm_tool("fetch_url")
            event.set_result(MessageEventResult().message("已关闭网页搜索功能"))
        else:
            event.set_result(
                MessageEventResult().message("操作参数错误，应为 on 或 off")
            )

    @llm_tool("web_search")
    async def search_from_search_engine(
        self, event: AstrMessageEvent, query: str
    ) -> str:
        """搜索网络以回答用户的问题。当用户需要搜索网络以获取即时性的信息时调用此工具。

        Args:
            query(string): 和用户的问题最相关的搜索关键词，用于在 Google 上搜索。
        """
        logger.info("web_searcher - search_from_search_engine: " + query)
        results = []
        RESULT_NUM = 5
        try:
            results = await self.google.search(query, RESULT_NUM)
        except Exception as e:
            logger.error(f"google search error: {e}, try the next one...")
        if len(results) == 0:
            logger.debug("search google failed")
            try:
                results = await self.bing_search.search(query, RESULT_NUM)
            except Exception as e:
                logger.error(f"bing search error: {e}, try the next one...")
        if len(results) == 0:
            logger.debug("search bing failed")
            try:
                results = await self.sogo_search.search(query, RESULT_NUM)
            except Exception as e:
                logger.error(f"sogo search error: {e}")
        if len(results) == 0:
            logger.debug("search sogo failed")
            return "没有搜索到结果"
        ret = ""
        idx = 1
        for i in results:
            logger.info(f"web_searcher - scraping web: {i.title} - {i.url}")
            try:
                site_result = await self._get_from_url(i.url)
            except BaseException:
                site_result = ""
            site_result = (
                site_result[:700] + "..." if len(site_result) > 700 else site_result
            )

            header = f"{idx}. {i.title} "

            if self.websearch_link and i.url:
                header += i.url

            ret += f"{header}\n{i.snippet}\n{site_result}\n\n"
            idx += 1

        if self.websearch_link:
            ret += "针对问题，请根据上面的结果分点总结，并且在结尾处附上对应内容的参考链接（如有）。"

        return ret

    @llm_tool("fetch_url")
    async def fetch_website_content(self, event: AstrMessageEvent, url: str) -> str:
        """fetch the content of a website with the given web url

        Args:
            url(string): The url of the website to fetch content from
        """
        resp = await self._get_from_url(url)
        return resp



import asyncio
import re
import wave
import uuid
import traceback
import os

from typing import AsyncGenerator
from astrbot.core.utils.io import download_file
from astrbot.core.utils.tencent_record_helper import wav_to_tencent_silk
from astrbot.api import logger
from astrbot.api.event import AstrMessageEvent, MessageChain
from astrbot.api.platform import AstrBotMessage, PlatformMetadata, Group, MessageMember
from astrbot.api.message_components import (
    Plain,
    Image,
    Record,
    At,
    File,
    Video,
    WechatEmoji as Emoji,
)
from .client import SimpleGewechatClient
from astrbot.core.utils.astrbot_path import get_astrbot_data_path


def get_wav_duration(file_path):
    with wave.open(file_path, "rb") as wav_file:
        file_size = os.path.getsize(file_path)
        n_channels, sampwidth, framerate, n_frames = wav_file.getparams()[:4]
        if n_frames == 2147483647:
            duration = (file_size - 44) / (n_channels * sampwidth * framerate)
        elif n_frames == 0:
            duration = (file_size - 44) / (n_channels * sampwidth * framerate)
        else:
            duration = n_frames / float(framerate)
        return duration


class GewechatPlatformEvent(AstrMessageEvent):
    def __init__(
        self,
        message_str: str,
        message_obj: AstrBotMessage,
        platform_meta: PlatformMetadata,
        session_id: str,
        client: SimpleGewechatClient,
    ):
        super().__init__(message_str, message_obj, platform_meta, session_id)
        self.client = client

    @staticmethod
    async def send_with_client(
        message: MessageChain, to_wxid: str, client: SimpleGewechatClient
    ):
        if not to_wxid:
            logger.error("无法获取到 to_wxid。")
            return

        #  检查@
        ats = []
        ats_names = []
        for comp in message.chain:
            if isinstance(comp, At):
                ats.append(comp.qq)
                ats_names.append(comp.name)
        has_at = False

        for comp in message.chain:
            if isinstance(comp, Plain):
                text = comp.text
                payload = {
                    "to_wxid": to_wxid,
                    "content": text,
                }
                if not has_at and ats:
                    ats = f"{','.join(ats)}"
                    ats_names = f"@{' @'.join(ats_names)}"
                    text = f"{ats_names} {text}"
                    payload["content"] = text
                    payload["ats"] = ats
                    has_at = True
                await client.post_text(**payload)

            elif isinstance(comp, Image):
                img_path = await comp.convert_to_file_path()
                # 为了安全，向 AstrBot 回调服务注册可被 gewechat 访问的文件，并获得文件 token
                token = await client._register_file(img_path)
                img_url = f"{client.file_server_url}/{token}"
                logger.debug(f"gewe callback img url: {img_url}")
                await client.post_image(to_wxid, img_url)
            elif isinstance(comp, Video):
                if comp.cover != "":
                    await client.forward_video(to_wxid, comp.cover)
                else:
                    try:
                        from pyffmpeg import FFmpeg
                    except (ImportError, ModuleNotFoundError):
                        logger.error(
                            "需要安装 pyffmpeg 库才能发送视频: pip install pyffmpeg"
                        )
                        raise ModuleNotFoundError(
                            "需要安装 pyffmpeg 库才能发送视频: pip install pyffmpeg"
                        )

                    video_url = comp.file
                    # 根据 url 下载视频
                    if video_url.startswith("http"):
                        video_filename = f"{uuid.uuid4()}.mp4"
                        temp_dir = os.path.join(get_astrbot_data_path(), "temp")
                        video_path = os.path.join(temp_dir, video_filename)
                        await download_file(video_url, video_path)
                    else:
                        video_path = video_url

                    video_token = await client._register_file(video_path)
                    video_callback_url = f"{client.file_server_url}/{video_token}"

                    # 获取视频第一帧
                    temp_dir = os.path.join(get_astrbot_data_path(), "temp")
                    thumb_path = os.path.join(
                        temp_dir, f"gewechat_video_thumb_{uuid.uuid4()}.jpg"
                    )

                    video_path = video_path.replace(" ", "\\ ")
                    try:
                        ff = FFmpeg()
                        command = f"-i {video_path} -ss 0 -vframes 1 {thumb_path}"
                        ff.options(command)
                        thumb_token = await client._register_file(thumb_path)
                        thumb_url = f"{client.file_server_url}/{thumb_token}"
                    except Exception as e:
                        logger.error(f"获取视频第一帧失败: {e}")

                    # 获取视频时长
                    try:
                        from pyffmpeg import FFprobe

                        # 创建 FFprobe 实例
                        ffprobe = FFprobe(video_url)
                        # 获取时长字符串
                        duration_str = ffprobe.duration
                        # 处理时长字符串
                        video_duration = float(duration_str.replace(":", ""))
                    except Exception as e:
                        logger.error(f"获取时长失败: {e}")
                        video_duration = 10

                    # 发送视频
                    await client.post_video(
                        to_wxid, video_callback_url, thumb_url, video_duration
                    )

                    # 删除临时缩略图文件
                    if os.path.exists(thumb_path):
                        os.remove(thumb_path)
            elif isinstance(comp, Record):
                # 默认已经存在 data/temp 中
                record_url = comp.file
                record_path = await comp.convert_to_file_path()

                temp_dir = os.path.join(get_astrbot_data_path(), "temp")
                silk_path = os.path.join(temp_dir, f"{uuid.uuid4()}.silk")
                try:
                    duration = await wav_to_tencent_silk(record_path, silk_path)
                except Exception as e:
                    logger.error(traceback.format_exc())
                    await client.post_text(to_wxid, f"语音文件转换失败。{str(e)}")
                logger.info("Silk 语音文件格式转换至: " + record_path)
                if duration == 0:
                    duration = get_wav_duration(record_path)
                token = await client._register_file(silk_path)
                record_url = f"{client.file_server_url}/{token}"
                logger.debug(f"gewe callback record url: {record_url}")
                await client.post_voice(to_wxid, record_url, duration * 1000)
            elif isinstance(comp, File):
                file_path = comp.file
                file_name = comp.name
                if file_path.startswith("file:///"):
                    file_path = file_path[8:]
                elif file_path.startswith("http"):
                    temp_dir = os.path.join(get_astrbot_data_path(), "temp")
                    temp_file_path = os.path.join(temp_dir, file_name)
                    await download_file(file_path, temp_file_path)
                    file_path = temp_file_path
                else:
                    file_path = file_path

                token = await client._register_file(file_path)
                file_url = f"{client.file_server_url}/{token}"
                logger.debug(f"gewe callback file url: {file_url}")
                await client.post_file(to_wxid, file_url, file_name)
            elif isinstance(comp, Emoji):
                await client.post_emoji(to_wxid, comp.md5, comp.md5_len, comp.cdnurl)
            elif isinstance(comp, At):
                pass
            else:
                logger.debug(f"gewechat 忽略: {comp.type}")

    async def send(self, message: MessageChain):
        to_wxid = self.message_obj.raw_message.get("to_wxid", None)
        await GewechatPlatformEvent.send_with_client(message, to_wxid, self.client)
        await super().send(message)

    async def get_group(self, group_id=None, **kwargs):
        # 确定有效的 group_id
        if group_id is None:
            group_id = self.get_group_id()

        if not group_id:
            return None

        res = await self.client.get_group(group_id)
        data: dict = res["data"]

        if not data["chatroomId"]:
            return None

        members = [
            MessageMember(user_id=member["wxid"], nickname=member["nickName"])
            for member in data.get("memberList", [])
        ]

        return Group(
            group_id=data["chatroomId"],
            group_name=data.get("nickName"),
            group_avatar=data.get("smallHeadImgUrl"),
            group_owner=data.get("chatRoomOwner"),
            members=members,
        )

    async def send_streaming(
        self, generator: AsyncGenerator, use_fallback: bool = False
    ):
        if not use_fallback:
            buffer = None
            async for chain in generator:
                if not buffer:
                    buffer = chain
                else:
                    buffer.chain.extend(chain.chain)
            if not buffer:
                return
            buffer.squash_plain()
            await self.send(buffer)
            return await super().send_streaming(generator, use_fallback)

        buffer = ""
        pattern = re.compile(r"[^。？！~…]+[。？！~…]+")

        async for chain in generator:
            if isinstance(chain, MessageChain):
                for comp in chain.chain:
                    if isinstance(comp, Plain):
                        buffer += comp.text
                        if any(p in buffer for p in "。？！~…"):
                            buffer = await self.process_buffer(buffer, pattern)
                    else:
                        await self.send(MessageChain(chain=[comp]))
                        await asyncio.sleep(1.5)  # 限速

        if buffer.strip():
            await self.send(MessageChain([Plain(buffer)]))
        return await super().send_streaming(generator, use_fallback)



from astrbot.core.db import BaseDatabase
from astrbot import logger
from astrbot.core.provider.func_tool_manager import FuncCall
from typing import List
from ..register import register_provider_adapter
from astrbot.core.provider.entities import LLMResponse
from .openai_source import ProviderOpenAIOfficial


@register_provider_adapter("zhipu_chat_completion", "智浦 Chat Completion 提供商适配器")
class ProviderZhipu(ProviderOpenAIOfficial):
    def __init__(
        self,
        provider_config: dict,
        provider_settings: dict,
        db_helper: BaseDatabase,
        persistant_history=True,
        default_persona=None,
    ) -> None:
        super().__init__(
            provider_config,
            provider_settings,
            db_helper,
            persistant_history,
            default_persona,
        )

    async def text_chat(
        self,
        prompt: str,
        session_id: str = None,
        image_urls: List[str] = None,
        func_tool: FuncCall = None,
        contexts=None,
        system_prompt=None,
        **kwargs,
    ) -> LLMResponse:
        if contexts is None:
            contexts = []
        new_record = await self.assemble_context(prompt, image_urls)
        context_query = []

        context_query = [*contexts, new_record]

        model_cfgs: dict = self.provider_config.get("model_config", {})
        model = self.get_model()
        # glm-4v-flash 只支持一张图片
        if model.lower() == "glm-4v-flash" and image_urls and len(context_query) > 1:
            logger.debug("glm-4v-flash 只支持一张图片，将只保留最后一张图片")
            logger.debug(context_query)
            new_context_query_ = []
            for i in range(0, len(context_query) - 1, 2):
                if isinstance(context_query[i].get("content", ""), list):
                    continue
                new_context_query_.append(context_query[i])
                new_context_query_.append(context_query[i + 1])
            new_context_query_.append(context_query[-1])  # 保留最后一条记录
            context_query = new_context_query_
            logger.debug(context_query)

        if system_prompt:
            context_query.insert(0, {"role": "system", "content": system_prompt})

        payloads = {"messages": context_query, **model_cfgs}
        try:
            llm_response = await self._query(payloads, func_tool)
            return llm_response
        except Exception as e:
            if "maximum context length" in str(e):
                retry_cnt = 10
                while retry_cnt > 0:
                    logger.warning(
                        f"请求失败：{e}。上下文长度超过限制。尝试弹出最早的记录然后重试。"
                    )
                    try:
                        self.pop_record(session_id)
                        llm_response = await self._query(payloads, func_tool)
                        break
                    except Exception as e:
                        if "maximum context length" in str(e):
                            retry_cnt -= 1
                        else:
                            raise e
            else:
                raise e



import os
import uuid
import ormsgpack
from pydantic import BaseModel, conint
from httpx import AsyncClient
from typing import Annotated, Literal
from ..provider import TTSProvider
from ..entities import ProviderType
from ..register import register_provider_adapter
from astrbot.core.utils.astrbot_path import get_astrbot_data_path


class ServeReferenceAudio(BaseModel):
    audio: bytes
    text: str


class ServeTTSRequest(BaseModel):
    text: str
    chunk_length: Annotated[int, conint(ge=100, le=300, strict=True)] = 200
    # 音频格式
    format: Literal["wav", "pcm", "mp3"] = "mp3"
    mp3_bitrate: Literal[64, 128, 192] = 128
    # 参考音频
    references: list[ServeReferenceAudio] = []
    # 参考模型 ID
    # 例如 https://fish.audio/m/7f92f8afb8ec43bf81429cc1c9199cb1/
    # 其中reference_id为 7f92f8afb8ec43bf81429cc1c9199cb1
    reference_id: str | None = None
    # 对中英文文本进行标准化，这可以提高数字的稳定性
    normalize: bool = True
    # 平衡模式将延迟减少到300毫秒，但可能会降低稳定性
    latency: Literal["normal", "balanced"] = "normal"


@register_provider_adapter(
    "fishaudio_tts_api", "FishAudio TTS API", provider_type=ProviderType.TEXT_TO_SPEECH
)
class ProviderFishAudioTTSAPI(TTSProvider):
    def __init__(
        self,
        provider_config: dict,
        provider_settings: dict,
    ) -> None:
        super().__init__(provider_config, provider_settings)
        self.chosen_api_key: str = provider_config.get("api_key", "")
        self.character: str = provider_config.get("fishaudio-tts-character", "可莉")
        self.api_base: str = provider_config.get(
            "api_base", "https://api.fish-audio.cn/v1"
        )
        self.headers = {
            "Authorization": f"Bearer {self.chosen_api_key}",
        }
        self.set_model(provider_config.get("model", None))

    async def _get_reference_id_by_character(self, character: str) -> str:
        """
        获取角色的reference_id

        Args:
            character: 角色名称

        Returns:
            reference_id: 角色的reference_id

        exception:
            APIException: 获取语音角色列表为空
        """
        sort_options = ["score", "task_count", "created_at"]
        async with AsyncClient(base_url=self.api_base.replace("/v1", "")) as client:
            for sort_by in sort_options:
                params = {"title": character, "sort_by": sort_by}
                response = await client.get(
                    "/model", params=params, headers=self.headers
                )
                resp_data = response.json()
                if resp_data["total"] == 0:
                    continue
                for item in resp_data["items"]:
                    if character in item["title"]:
                        return item["_id"]
            return None

    async def _generate_request(self, text: str) -> dict:
        return ServeTTSRequest(
            text=text,
            format="wav",
            reference_id=await self._get_reference_id_by_character(self.character),
        )

    async def get_audio(self, text: str) -> str:
        temp_dir = os.path.join(get_astrbot_data_path(), "temp")
        path = os.path.join(temp_dir, f"fishaudio_tts_api_{uuid.uuid4()}.wav")
        self.headers["content-type"] = "application/msgpack"
        request = await self._generate_request(text)
        async with AsyncClient(base_url=self.api_base).stream(
            "POST",
            "/tts",
            headers=self.headers,
            content=ormsgpack.packb(request, option=ormsgpack.OPT_SERIALIZE_PYDANTIC),
        ) as response:
            if response.headers["content-type"] == "audio/wav":
                with open(path, "wb") as f:
                    async for chunk in response.aiter_bytes():
                        f.write(chunk)
                return path
            text = await response.aread()
            raise Exception(f"Fish Audio API请求失败: {text}")



import base64
import wave
import os
from io import BytesIO
import asyncio
import tempfile
from astrbot.core.utils.astrbot_path import get_astrbot_data_path


async def tencent_silk_to_wav(silk_path: str, output_path: str) -> str:
    import pysilk

    with open(silk_path, "rb") as f:
        input_data = f.read()
        if input_data.startswith(b"\x02"):
            input_data = input_data[1:]
        input_io = BytesIO(input_data)
        output_io = BytesIO()
        pysilk.decode(input_io, output_io, 24000)
        output_io.seek(0)
        with wave.open(output_path, "wb") as wav:
            wav.setnchannels(1)
            wav.setsampwidth(2)
            wav.setframerate(24000)
            wav.writeframes(output_io.read())

    return output_path


async def wav_to_tencent_silk(wav_path: str, output_path: str) -> int:
    """返回 duration"""
    try:
        import pilk
    except (ImportError, ModuleNotFoundError) as _:
        raise Exception(
            "pilk 模块未安装，请前往管理面板->控制台->安装pip库 安装 pilk 这个库"
        )
    # with wave.open(wav_path, 'rb') as wav:
    #     wav_data = wav.readframes(wav.getnframes())
    #     wav_data = BytesIO(wav_data)
    #     output_io = BytesIO()
    #     pysilk.encode(wav_data, output_io, 24000, 24000)
    #     output_io.seek(0)

    #     # 在首字节添加 \x02,去除结尾的\xff\xff
    #     silk_data = output_io.read()
    #     silk_data_with_prefix = b'\x02' + silk_data[:-2]

    #     # return BytesIO(silk_data_with_prefix)
    #     with open(output_path, "wb") as f:
    #         f.write(silk_data_with_prefix)

    #     return 0
    with wave.open(wav_path, "rb") as wav:
        rate = wav.getframerate()
        duration = pilk.encode(wav_path, output_path, pcm_rate=rate, tencent=True)
        return duration


async def wav_to_tencent_silk_base64(wav_path: str) -> str:
    """
    将 WAV 文件转为 Silk，并返回 Base64 字符串。
    默认采样率为 24000，输出临时文件为 temp/output.silk。

    参数:
    - wav_path: 输入 .wav 文件路径（需为 PCM 16bit）

    返回:
    - Base64 编码的 Silk 字符串
    - duration: 音频时长（秒）
    """
    try:
        import pilk
    except ImportError as e:
        raise Exception("pysilk 模块未安装，请安装 pysilk") from e

    temp_dir = os.path.join(get_astrbot_data_path(), "temp")
    os.makedirs(temp_dir, exist_ok=True)

    with wave.open(wav_path, "rb") as wav:
        rate = wav.getframerate()

    with tempfile.NamedTemporaryFile(
        suffix=".silk", delete=False, dir=temp_dir
    ) as tmp_file:
        silk_path = tmp_file.name

    try:
        duration = await asyncio.to_thread(
            pilk.encode, wav_path, silk_path, pcm_rate=rate, tencent=True
        )

        with open(silk_path, "rb") as f:
            silk_bytes = await asyncio.to_thread(f.read)
            silk_b64 = base64.b64encode(silk_bytes).decode("utf-8")

        return silk_b64, duration  # 已是秒
    finally:
        if os.path.exists(silk_path):
            os.remove(silk_path)



from defusedxml import ElementTree as eT
from astrbot.api import logger
from astrbot.api.message_components import (
    WechatEmoji as Emoji,
    Plain,
    Image,
    BaseMessageComponent,
)


class GeweDataParser:
    def __init__(
        self,
        content: str,
        is_private_chat: bool = False,
        cached_texts=None,
        cached_images=None,
        raw_message: dict = None,
        downloader=None,
    ):
        self._xml = None
        self.content = content
        self.is_private_chat = is_private_chat
        self.cached_texts = cached_texts or {}
        self.cached_images = cached_images or {}
        self.downloader = downloader

        raw_message = raw_message or {}
        self.from_user_name = raw_message.get("from_user_name", {}).get("str", "")
        self.to_user_name = raw_message.get("to_user_name", {}).get("str", "")
        self.msg_id = raw_message.get("msg_id", "")

    def _format_to_xml(self):
        if self._xml:
            return self._xml

        try:
            msg_str = self.content
            if not self.is_private_chat:
                parts = self.content.split(":\n", 1)
                msg_str = parts[1] if len(parts) == 2 else self.content

            self._xml = eT.fromstring(msg_str)
            return self._xml
        except Exception as e:
            logger.error(f"[XML解析失败] {e}")
            raise

    async def parse_mutil_49(self) -> list[BaseMessageComponent] | None:
        """
        处理 msg_type == 49 的多种 appmsg 类型（目前支持 type==57）
        """
        try:
            appmsg_type = self._format_to_xml().findtext(".//appmsg/type")
            if appmsg_type == "57":
                return await self.parse_reply()
        except Exception as e:
            logger.warning(f"[parse_mutil_49] 解析失败: {e}")
        return None

    async def parse_reply(self) -> list[BaseMessageComponent]:
        """
        处理 type == 57 的引用消息：支持文本（1）、图片（3）、嵌套49（49）
        """
        components = []

        try:
            appmsg = self._format_to_xml().find("appmsg")
            if appmsg is None:
                return [Plain("[引用消息解析失败]")]

            refermsg = appmsg.find("refermsg")
            if refermsg is None:
                return [Plain("[引用消息解析失败]")]

            quote_type = int(refermsg.findtext("type", "0"))
            nickname = refermsg.findtext("displayname", "未知发送者")
            quote_content = refermsg.findtext("content", "")
            svrid = refermsg.findtext("svrid")

            match quote_type:
                case 1:  # 文本引用
                    quoted_text = self.cached_texts.get(str(svrid), quote_content)
                    components.append(Plain(f"[引用] {nickname}: {quoted_text}"))

                case 3:  # 图片引用
                    quoted_image_b64 = self.cached_images.get(str(svrid))
                    if not quoted_image_b64:
                        try:
                            quote_xml = eT.fromstring(quote_content)
                            img = quote_xml.find("img")
                            cdn_url = (
                                img.get("cdnbigimgurl") or img.get("cdnmidimgurl")
                                if img is not None
                                else None
                            )
                            if cdn_url and self.downloader:
                                image_resp = await self.downloader(
                                    self.from_user_name, self.to_user_name, self.msg_id
                                )
                                quoted_image_b64 = (
                                    image_resp.get("Data", {})
                                    .get("Data", {})
                                    .get("Buffer")
                                )
                        except Exception as e:
                            logger.warning(f"[引用图片解析失败] svrid={svrid} err={e}")

                    if quoted_image_b64:
                        components.extend(
                            [
                                Image.fromBase64(quoted_image_b64),
                                Plain(f"[引用] {nickname}: [引用的图片]"),
                            ]
                        )
                    else:
                        components.append(
                            Plain(f"[引用] {nickname}: [引用的图片 - 未能获取]")
                        )

                case 49:  # 嵌套引用
                    try:
                        nested_root = eT.fromstring(quote_content)
                        nested_title = nested_root.findtext(".//appmsg/title", "")
                        components.append(Plain(f"[引用] {nickname}: {nested_title}"))
                    except Exception as e:
                        logger.warning(f"[嵌套引用解析失败] err={e}")
                        components.append(Plain(f"[引用] {nickname}: [嵌套引用消息]"))

                case _:  # 其他未识别类型
                    logger.info(f"[未知引用类型] quote_type={quote_type}")
                    components.append(Plain(f"[引用] {nickname}: [不支持的引用类型]"))

            # 主消息标题
            title = appmsg.findtext("title", "")
            if title:
                components.append(Plain(title))

        except Exception as e:
            logger.error(f"[parse_reply] 总体解析失败: {e}")
            return [Plain("[引用消息解析失败]")]

        return components

    def parse_emoji(self) -> Emoji | None:
        """
        处理 msg_type == 47 的表情消息（emoji）
        """
        try:
            emoji_element = self._format_to_xml().find(".//emoji")
            if emoji_element is not None:
                return Emoji(
                    md5=emoji_element.get("md5"),
                    md5_len=emoji_element.get("len"),
                    cdnurl=emoji_element.get("cdnurl"),
                )
        except Exception as e:
            logger.error(f"[parse_emoji] 解析失败: {e}")

        return None
